import time
import logging

class Intent:
    def __init__(self, name, driver):
        self.logger = logging.getLogger('snips_helper')
        self.logger.debug("Creating intent: {}".format(name))
        self.__name = name
        self.__driver = driver

    @property
    def name(self):
        return self.__name

    @property
    def driver(self):
        return self.__driver

    def utterances(self):
        self.logger.debug("Getting utterances")
        self.__activate()
        self.__get_utterance_dot_button().click()
        self.__get_export_utterance_button().click()
        training_data = self.__get_modal_text_area().text.split("\n")
        self.__get_modal_close_button().click()
        self.__deactivate()
        return training_data

    def slots(self):
        self.logger.debug("Getting slots")
        self.__activate()
        self.__get_slots_dot_button().click()
        self.__get_export_slots_button().click()
        slots = self.__get_modal_text_area().text.split("\n")
        self.__get_modal_close_button().click()
        self.__deactivate()
        if slots == ['']: slots = []
        return slots

    def export_slot_values(self, slot):
        self.logger.debug("Getting slot: {}".format(slot))
        self.__activate()
        if self.__get_slot_editor_button(slot):
            self.__get_slot_editor_button(slot).click()
        else:
            self.logger.debug("No slots found {}".format(self.__get_slot_editor_button(slot)))
            self.__deactivate()
            return None
        self.__get_slot_modal_export_button().click()
        slot_values = self.__get_modal_text_area().text.split("\n")
        self.__get_slot_modal_dialog_close_button().click()
        self.__get_slot_modal_close_button().click()
        self.__deactivate()
        return slot_values

    def import_slot_values(self, slot, slot_values):
        self.logger.debug("Getting slot: {}".format(slot))
        existing_slots = [x + '\n' for x in self.export_slot_values(slot)]
        slots = (existing_slots +
                 list(set(slot_values) - set(existing_slots)))
        self.__activate()
        if self.__get_slot_editor_button(slot):
            self.__get_slot_editor_button(slot).click()
        else:
            self.logger.debug("No slots found {}".format(
                self.__get_slot_editor_button(slot)))
            self.__deactivate()
            return None
        self.__get_slot_modal_import_button().click()
        self.__get_modal_text_area().send_keys(slots)
        self.__get_modal_import_button().click()
        self.__get_slot_modal_close_button().click()
        self.__deactivate()
        return slots

    def import_utterances(self, utterances):
        self.logger.debug("Importing utterances")
        existing_utterances = [x + '\n' for x in self.utterances()]
        imported_utterances = (existing_utterances +
                               list(set(utterance) - set(existing_utterances)))
        self.__activate()
        self.__get_utterance_dot_button().click()
        self.__get_import_utterance_button().click()
        self.__get_modal_text_area().send_keys(utterances)
        self.__get_modal_import_button().click()
        time.sleep(1)
        self.save()
        self.__deactivate()
        return imported_utterances

    def delete_utterances(self):
        self.logger.debug("Deleting utterances")
        self.__activate()
        while self.__get_utterance_delete_button():
            self.__get_utterance_delete_button().click()
        self.save()
        return

    def save(self):
        self.logger.debug("Saving intent: {}".format(self.name))
        if not self.__get_save_button():
            self.__activate()
        self.__get_save_button().click()
        time.sleep(1)
        self.__deactivate()

    def __activate(self):
        self.logger.debug("Activating intent: {}".format(self.name))
        self.__get_intent().click()

    def __deactivate(self):
        self.logger.debug("Deactivating intent: {}".format(self.name))
        self.__get_back_to_assistant_button().click()

    def __get_intent(self):
        self.logger.debug("Getting intent: {}".format(self.name))
        selector1 = "//*[contains(@class, 'assistant-intent__title')]"
        selector2 = "[text()='{}']".format(self.name)
        selector = selector1 + selector2
        return self.driver.find_element_by_xpath(selector)

    def __get_back_to_assistant_button(self):
        self.logger.debug("Get back to assistant button")
        return self.driver.find_element_by_link_text('Back to Assistant')

    def __get_save_button(self):
        self.logger.debug("Get save button")
        return self.driver.find_element_by_class_name('save-intent-button')

    def __get_export_headers(self):
        self.logger.debug("Get export headers")
        return self.driver.find_elements_by_class_name(
            'intent-editor-section-header')

    def __get_import_headers(self):
        self.logger.debug("Get import headers")
        return self.driver.find_elements_by_class_name(
            'intent-editor-section-header')

    def __get_utterance_section(self):
        self.logger.debug("Get utterance section")
        return self.__get_export_headers()[1]

    def __get_utterance_delete_button(self):
        self.logger.debug("Get utterance delete button")
        selector = 'button--unstyled'
        utterance_section = self.__get_utterance_section()
        return utterance_section.find_element_by_class_name(selector)

    def __get_utterance_dot_button(self):
        self.logger.debug("Get utterance dot button")
        utterance_section = self.__get_utterance_section()
        return utterance_section.find_element_by_class_name('dots-icon-button')

    def __get_export_utterance_button(self):
        self.logger.debug("Get export utterance button")
        selector = "//*[text()='Export Training Examples']"
        return self.driver.find_element_by_xpath(selector)

    def __get_import_utterance_button(self):
        self.logger.debug("Get import utterance button")
        selector = "//*[text()='Import Training Examples']"
        return self.driver.find_element_by_xpath(selector)

    def __get_slots_section(self):
        self.logger.debug("Get slots section")
        return self.__get_export_headers()[0]

    def __get_slots_dot_button(self):
        self.logger.debug("Get slots dot button")
        slot_section = self.__get_slots_section()
        return slot_section.find_element_by_class_name('dots-icon-button')

    def __get_export_slots_button(self):
        self.logger.debug("Get export slots button")
        selector = "//*[text()='Export Slots']"
        return self.driver.find_element_by_xpath(selector)

    def __get_import_slots_button(self):
        self.logger.debug("Get import slots button")
        selector = "//*[text()='Import Slots']"
        return self.driver.find_element_by_xpath(selector)

    def __get_slots_island(self):
        self.logger.debug("Get slot island")
        return self.driver.find_elements_by_class_name('island--with-row')[0]

    def __get_slot_editors(self):
        self.logger.debug("Get slots editors")
        slots_island = self.__get_slots_island()
        return slots_island.find_elements_by_class_name('slot-editor')

    def __get_slot_editor_row(self, slot):
        self.logger.debug("Get slot editor row")
        for editor in self.__get_slot_editors():
            slot_name = editor.find_element_by_class_name(
                'slot-editor__name').get_attribute('value')
            if slot == slot_name:
                return editor

    def __get_slot_editor_button(self, slot):
        self.logger.debug("Get slot editor button")
        editor_row = self.__get_slot_editor_row(slot)
        if not editor_row:
            return None
        if editor_row.find_element_by_class_name(
                'item-option__subtitle').text == 'builtin':
            return None
        editor_button = editor_row.find_element_by_class_name(
                'slot-editor--action-button')
        self.driver.execute_script(
            "arguments[0].setAttribute('style','visibility:visible;');",
            editor_button)
        return editor_button

    def __get_slot_modal(self):
        self.logger.debug("Get slot modal")
        return self.driver.find_element_by_class_name(
            'fullscreen-modal')

    def __get_slot_modal_content(self):
        self.logger.debug("Get slot modal content")
        modal = self.__get_slot_modal()
        return modal.find_element_by_class_name('fullscreen-modal__content')

    def __get_slot_modal_export_button(self):
        self.logger.debug("Get slot modal export button")
        modal_content = self.__get_slot_modal_content()
        export_button = modal_content.find_element_by_xpath("//*[text()=' Export']")
        self.driver.execute_script(
                "return arguments[0].scrollIntoView();", export_button)
        return export_button

    def __get_slot_modal_import_button(self):
        self.logger.debug("Get slot modal import button")
        modal_content = self.__get_slot_modal_content()
        import_button = modal_content.find_element_by_xpath("//*[text()=' Import']")
        self.driver.execute_script(
                "return arguments[0].scrollIntoView();", import_button)
        return import_button

    def __get_slot_modal_dialog_close_button(self):
        modal = self.__get_modal()
        return modal.find_element_by_class_name("modal-dialog__closeundefined")

    def __get_slot_modal_close_button(self):
        modal = self.__get_slot_modal()
        close_button = modal.find_element_by_xpath("//*[text()='Close']")
        self.driver.execute_script(
                "return arguments[0].scrollIntoView();", close_button)
        return close_button

    def __get_modal(self):
        self.logger.debug("Get modal")
        return self.driver.find_element_by_class_name(
            'modal-dialog__dialog')

    def __get_modal_content(self):
        self.logger.debug("Get modal content")
        modal = self.__get_modal()
        return modal.find_element_by_class_name('modal-dialog__content')

    def __get_modal_text_area(self):
        self.logger.debug("Get text area")
        modal = self.__get_modal_content()
        return modal.find_element_by_tag_name('textarea')

    def __get_modal_close_button(self):
        self.logger.debug("Get modal close button")
        modal = self.__get_modal()
        return modal.find_element_by_xpath("//*[text()='Close']")

    def __get_modal_import_button(self):
        self.logger.debug("Get modal import button")
        modal = self.__get_modal()
        return modal.find_element_by_xpath("//*[text()='Import']")



from .intent import Intent
import time
import logging


class Bundle:
    def __init__(self, name, driver):
        self.logger = logging.getLogger('snips_helper')
        self.logger.debug("Creating bundle: {}".format(name))
        self.__name = name
        self.__driver = driver

    @property
    def name(self):
        return self.__name

    @property
    def driver(self):
        return self.__driver

    def __get_bundle(self):
        selector1 = "//*[contains(@class, 'bundle-header__title')]"
        selector2 = "[@title='{}']".format(self.name)
        selector = selector1 + selector2
        return self.driver.find_element_by_xpath(selector)

    def __activate(self):
        self.logger.debug("Selecting bundle: {}".format(self.name))
        self.__get_bundle().click()

    def get_intents(self):
        self.logger.debug("Getting intents for: {}".format(self.name))
        self.__activate()
        intent_elements = self.driver.find_elements_by_class_name(
            'assistant-intent__title')

        intents = []
        for intent in intent_elements:
            intents.append(Intent(intent.text, self.driver))

        return intents

    def create_intent(self, intent, slots, utterances):
        self.logger.debug("Creating new intent: {}".format(intent))

        for existing_intent in self.get_intents():
            if existing_intent.name == intent:
                self.logger.debug("intent exists: {}".format(intent))
                return None
        self.__get_create_intent_button().click()
        self.__set_intent_name().send_keys(intent)
        self.__get_utterance_dot_button().click()
        self.__get_import_utterance_button().click()
        self.__get_modal_text_area().send_keys('\n'.join(utterances))
        self.__get_modal_import_button().click()
        time.sleep(1)
        self.save_intent()
        return intent

    def save_intent(self):
        self.logger.debug("Saving intent")
        self.__get_save_button().click()
        time.sleep(1)
        self.__deactivate()

    def __deactivate(self):
        self.logger.debug("Deactivating intent")
        self.__get_back_to_assistant_button().click()

    def __get_back_to_assistant_button(self):
        self.logger.debug("Get back to assistant button")
        return self.driver.find_element_by_link_text('Back to Assistant')

    def __get_save_button(self):
        self.logger.debug("Get save button")
        return self.driver.find_element_by_class_name('save-intent-button')

    def __get_create_intent_button(self):
        self.logger.debug("Get create intent button")
        return self.driver.find_element_by_xpath("//*[text()='Create new Intent']")

    def __set_intent_name(self):
        self.logger.debug("Get name input button")
        return self.driver.find_element_by_name('intentName')

    def __get_import_headers(self):
        self.logger.debug("Get import headers")
        return self.driver.find_elements_by_class_name(
            'intent-editor-section-header')

    def __get_utterance_section(self):
        self.logger.debug("Get utterance section")
        return self.__get_import_headers()[1]

    def __get_utterance_dot_button(self):
        self.logger.debug("Get utterance dot button")
        utterance_section = self.__get_utterance_section()
        return utterance_section.find_element_by_class_name('dots-icon-button')

    def __get_import_utterance_button(self):
        self.logger.debug("Get import utterance button")
        selector = "//*[text()='Import Training Examples']"
        return self.driver.find_element_by_xpath(selector)

    def __get_modal(self):
        self.logger.debug("Get modal")
        return self.driver.find_element_by_class_name(
            'modal-dialog__dialog')

    def __get_modal_content(self):
        self.logger.debug("Get modal content")
        modal = self.__get_modal()
        return modal.find_element_by_class_name('modal-dialog__content')

    def __get_modal_text_area(self):
        self.logger.debug("Get text area")
        modal = self.__get_modal_content()
        return modal.find_element_by_tag_name('textarea')

    def __get_modal_import_button(self):
        self.logger.debug("Get modal import button")
        modal = self.__get_modal()
        return modal.find_element_by_xpath("//*[text()='Import']")



import logging
import json
from .slot_parser import SlotParser

class IntentParser:

    def __init__(self, data):
        self.logger = logging.getLogger('snips_helper')
        self.__raw_data = data
        self.__parsed_data = self.parse_data(self.__raw_data)

    def parse_data(self, data):
        parsed_data = json.loads(data)
        self.__session_id = parsed_data['sessionId']
        self.__custom_data = parsed_data['customData']
        self.__site_id = parsed_data['siteId']
        self.__input_text = parsed_data['input']
        self.__name = parsed_data['intent']['intentName']
        self.__probability = parsed_data['intent']['probability']

        slots = parsed_data['slots']
        parsed_slots = []
        for slot in slots:
            parsed_slots.append(SlotParser(slot))

        self.__slots = parsed_slots

        return parsed_data


    @property
    def name(self):
        return self.__name

    @property
    def raw_data(self):
        return self.__raw_data

    @property
    def parsed_data(self):
        return self.__parsed_data

    @property
    def session_id(self):
        return self.__session_id

    @property
    def custom_data(self):
        return self.__custom_data

    @property
    def site_id(self):
        return self.__site_id

    @property
    def input_text(self):
        return self.__input_text

    @property
    def probability(self):
        return self.__probability

    @property
    def slots(self):
        return self.__slots





import logging
import json

class SlotParser:
    def __init__(self, slot_data):
        self.__raw_value = slot_data['rawValue']
        self.__value = slot_data['value']['value']
        self.__name = slot_data['slotName']

    @property
    def raw_value(self):
        return self.__raw_value

    @property
    def value(self):
        return self.__value

    @property
    def name(self):
        return self.__name


from setuptools import setup

setup(name='snips_helper',
      version='0.5',
      description='Snips.ai Helper',
      url='http://github.com/uchagani/snips_helper',
      author='Umair Chagani',
      author_email='umair.chagani@gmail.com',
      license='MIT',
      packages=['snips_helper'],
      install_requires=['selenium'],
      zip_safe=False)


from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
import os
from .bundle import Bundle
import logging


class ConsoleHelper:
    LOGIN_URL = 'https://console.snips.ai/login'
    IMPLICIT_DELAY = 10

    def __init__(self, chrome_driver_path='/usr/bin/chromedriver',
                 download_dir=None, headless=True, log_level='WARNING'):

        self.logger = logging.getLogger('snips_helper')

        numeric_level = getattr(logging, log_level.upper(), None)
        if not isinstance(numeric_level, int):
            raise ValueError('Invalid log level: %s' % log_level)

        self.logger.setLevel(level=numeric_level)
        ch = logging.StreamHandler()
        ch.setLevel(numeric_level)
        self.logger.addHandler(ch)
        
        if download_dir is None:
            download_dir = os.getcwd()

        self.chrome_options = Options()
        if headless:
            self.chrome_options.add_argument("--headless")
        self.chrome_options.add_argument("--disable-gpu")
        self.chrome_options.add_argument("--no-sandbox")
        self.chrome_options.add_experimental_option("prefs", {
            "download.default_directory": download_dir,
            "download.prompt_for_download": False,
            "download.directory_upgrade": True,
            "safebrowsing.enabled": True
        })

        self.logger.debug("Creating chrome driver")
        self.driver = webdriver.Chrome(chrome_options=self.chrome_options,
                                       executable_path=chrome_driver_path)

        self.logger.debug("Sending command to enable headless downloads")
        self.driver.command_executor._commands["send_command"] = (
            "POST", '/session/$sessionId/chromium/send_command')
        params = {'cmd': 'Page.setDownloadBehavior',
                  'params': {'behavior': 'allow',
                             'downloadPath': download_dir}}

        self.driver.execute("send_command", params)
        self.driver.implicitly_wait(self.IMPLICIT_DELAY)

        self.logger.debug("Setting window size to maximize")
        self.driver.set_window_position(0,0)
        self.driver.set_window_size(4096, 2160)
        self.driver.maximize_window()

    def login(self, email, password, assistant=None):
        self.logger.debug("Navigating to Snips Console")
        self.driver.get(self.LOGIN_URL)
        self.handle_cookie_message()
        self.logger.debug("Getting email field")
        email_field = self.driver.find_element_by_name('email')
        self.logger.debug("Setting email field")
        email_field.send_keys(email)
        self.logger.debug("Getting password field")
        password_field = self.driver.find_element_by_name('password')
        self.logger.debug("Setting password field")
        password_field.send_keys(password)
        self.logger.debug("Submitting login")
        password_field.submit()

        if assistant is not None:
            self.change_assistant(assistant)

    def quit(self):
        self.driver.quit()

    def change_assistant(self, assistant):
        self.logger.debug("Changing assistant")
        assistant_selector = self.driver.find_element_by_class_name(
            "header-assistants-select")
        assistant_selector.click()
        WebDriverWait(self.driver, 5).until(EC.visibility_of_element_located((By.CLASS_NAME, "Select-menu-outer")))
        assistant_options = self.driver.find_element_by_class_name(
            "Select-menu-outer")
        assistant = assistant_options.find_element_by_css_selector(
            "div[aria-label='{}']".format(assistant))
        assistant.click()

    def get_assistant(self):
        self.logger.debug("Getting assistant")
        assistant_label = self.driver.find_element_by_class_name(
            "header-assistants-select__label").text
        return assistant_label

    def handle_cookie_message(self):
        try:
            cookies_button = self.driver.find_element_by_class_name(
                'cookies-usage-info__ok-button')
            self.logger.debug("Handling cookie message")
            cookies_button.click()
        except:
            return

    def get_header(self):
        return self.driver.find_element_by_class_name(
            'header-assistants-select__label')

    def get_bundles(self):
        self.logger.debug("Changing bundles")
        bundle_elements = self.driver.find_elements_by_class_name(
            'bundle-header__title')
        bundles = []
        for bundle_element in bundle_elements:
            bundles.append(Bundle(bundle_element.text, self.driver))

        return bundles

    def retrain_assistant(self):
        self.logger.debug("Retraining assistant")
        self.get_header().click()
        bundle = self.get_bundles()[0]
        intent = bundle.get_intents()[0]
        intent.save()

    def get_download_button(self, timeout):
        self.logger.debug("Getting download button")
        return WebDriverWait(self.driver, timeout).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, "[download]")))

    def get_download_close_button(self, timeout):
        self.logger.debug("Getting download close button")
        download_modal = WebDriverWait(self.driver, timeout).until(
            EC.presence_of_element_located(
                (By.CLASS_NAME, "download-assistant-modal")))

        return WebDriverWait(download_modal, timeout).until(
            EC.element_to_be_clickable(
                (By.XPATH, "//*[text()='Close']")))

    def download_assistant(self, timeout=120):
        self.logger.debug("Downloading assistant")
        self.get_download_button(timeout).click()
        self.get_download_close_button(timeout).click()
